@@ -3,7 +3,7 @@
     Author:        Keri Harris
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2025, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -81,6 +81,8 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_QLF_BOOT),
   DEBUG_TOPIC(MSG_QLF_BOOT_READ),
   DEBUG_TOPIC(MSG_QLF_LABEL),
+  DEBUG_TOPIC(MSG_QLF_IMPORT),
+  DEBUG_TOPIC(MSG_QLF_INCLUDE),
   DEBUG_TOPIC(MSG_PROC_COUNT),
   DEBUG_TOPIC(MSG_CUT),
 
@@ -119,6 +121,8 @@ const debug_topic debug_topics[] =
 
   DEBUG_TOPIC(MSG_BACKTRACK),
   DEBUG_TOPIC(MSG_CSTACK),
+  DEBUG_TOPIC(MSG_GMP_ALLOC),
+  DEBUG_TOPIC(MSG_GMP_OVERFLOW),
 
 						/* GC messages */
   DEBUG_TOPIC(MSG_AGC),
@@ -176,6 +180,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_JIT),
   DEBUG_TOPIC(MSG_JIT_DELINDEX),
   DEBUG_TOPIC(MSG_JIT_POOR),
+  DEBUG_TOPIC(MSG_JIT_PRIMARY),
 
   DEBUG_TOPIC(MSG_RECONSULT),
   DEBUG_TOPIC(MSG_RECONSULT_PRED),
@@ -234,6 +239,8 @@ const debug_topic debug_topics[] =
 
   DEBUG_TOPIC(MSG_READ_OP),
 
+  DEBUG_TOPIC(MSG_WRITE_FLOAT),
+
   DEBUG_TOPIC(CHK_SECURE),
   DEBUG_TOPIC(CHK_HIGH_ARITY),
   DEBUG_TOPIC(CHK_HIGHER_ADDRESS),
@@ -280,12 +287,12 @@ static int
 prolog_debug_topic(const char *topic, int flag)
 { long level;
   char *end;
-  int success = FALSE;
+  int success = false;
 
   level = strtol(topic, &end, 10);
   if ( end > topic && *end == EOS )
   { GD->debug_level = level;
-    success = TRUE;
+    success = true;
   } else
   { const debug_topic *dt = NULL;
 
@@ -299,7 +306,7 @@ prolog_debug_topic(const char *topic, int flag)
 
     while ( (dt = get_next_debug_topic(topic, dt)) )
     { int code = dt->code;
-      success = TRUE;
+      success = true;
 
       if ( code <= DBG_LEVEL9 )
 	GD->debug_level = code;
@@ -349,7 +356,7 @@ prolog_debug_from_string(const char *spec, int flag)
       buf[end-spec] = EOS;
       if ( !prolog_debug_topic(buf, flag) )
       { if (quiet)
-	  return FALSE;
+	  return false;
 	Sdprintf("ERROR: Unknown debug topic: %s\n", buf);
 	PL_halt(1);
       }
@@ -362,24 +369,24 @@ prolog_debug_from_string(const char *spec, int flag)
 
   if ( !prolog_debug_topic(spec, flag) )
   { if (quiet)
-      return FALSE;
+      return false;
     Sdprintf("ERROR: Unknown debug topic: %s\n", spec);
     PL_halt(1);
   }
 
-  return TRUE;
+  return true;
 }
 
 
 int
 PL_prolog_debug(const char *topic)
-{ return prolog_debug_topic(topic, TRUE);
+{ return prolog_debug_topic(topic, true);
 }
 
 
 int
 PL_prolog_nodebug(const char *topic)
-{ return prolog_debug_topic(topic, FALSE);
+{ return prolog_debug_topic(topic, false);
 }
 
 
@@ -392,7 +399,7 @@ prolog_debug(term_t t, int flag)
     fail;
 
   if ( prolog_debug_topic(topic, flag) )
-    return TRUE;
+    return true;
 
   return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_debug_topic, t);
 }
@@ -400,13 +407,13 @@ prolog_debug(term_t t, int flag)
 
 static
 PRED_IMPL("prolog_debug", 1, pl_prolog_debug, 0)
-{ return prolog_debug(A1, TRUE);
+{ return prolog_debug(A1, true);
 }
 
 
 static
 PRED_IMPL("prolog_nodebug", 1, pl_prolog_nodebug, 0)
-{ return prolog_debug(A1, FALSE);
+{ return prolog_debug(A1, false);
 }
 
 
