@@ -3,7 +3,7 @@
     Author:        Keri Harris
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2025, University of Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -81,6 +81,8 @@
 #define MSG_QLF_BOOT		 36
 #define MSG_QLF_BOOT_READ	 37
 #define MSG_QLF_LABEL		 38
+#define MSG_QLF_IMPORT		 92
+#define MSG_QLF_INCLUDE		 93
 #define MSG_PROC_COUNT		 39
 #define MSG_CUT			 40
 
@@ -118,6 +120,9 @@
 #define MSG_PRED_COUNT		 83
 
 #define MSG_BACKTRACK		 90
+#define MSG_CSTACK		 91
+#define MSG_GMP_ALLOC		 92
+#define MSG_GMP_OVERFLOW	 93
 
 						/* GC messages */
 #define	MSG_AGC			 100
@@ -175,6 +180,7 @@
 #define MSG_JIT			 180
 #define MSG_JIT_DELINDEX	 181
 #define MSG_JIT_POOR		 182
+#define MSG_JIT_PRIMARY		 183
 
 #define MSG_RECONSULT		 190
 #define MSG_RECONSULT_PRED	 191
@@ -233,6 +239,8 @@
 
 #define MSG_READ_OP		 362
 
+#define MSG_WRITE_FLOAT		 370
+
 #define CHK_SECURE              1000
 #define CHK_HIGH_ARITY          1001
 #define CHK_HIGHER_ADDRESS      1002
