@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2024, University of Amsterdam
+    Copyright (c)  1985-2025, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -256,7 +256,8 @@ raiseSignal(PL_local_data_t *ld, int sig)
 
     do
     { alerted = ld->alerted;
-    } while ( !COMPARE_AND_SWAP_INT(&ld->alerted, alerted, alerted|ALERT_SIGNAL) );
+    } while ( !COMPARE_AND_SWAP_INT(&ld->alerted, alerted,
+				    alerted|ALERT_SIGNAL) );
 
     return true;
   }
@@ -268,7 +269,7 @@ raiseSignal(PL_local_data_t *ld, int sig)
 int
 pendingSignal(PL_local_data_t *ld, int sig)
 { if ( IS_VALID_SIGNAL(sig) && ld )
-  { return WSIGMASK_ISSET(ld->signal.pending, sig) ? true : false;
+  { return WSIGMASK_ISSET(ld->signal.pending, sig);
   }
 
   return -1;
@@ -1440,10 +1441,10 @@ default_action:
 Trail a raw pointer after we know there is insufficient tail space.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+bool
 grow_trail_ptr(DECL_LD Word p)
 { PushPtr(p);
-  int rc = ensureGlobalSpace(0, ALLOW_GC);
+  bool rc = ensureGlobalSpace(0, ALLOW_GC);
   PopPtr(p);
   if ( !rc )
     return false;
@@ -1516,7 +1517,7 @@ reclaim_attvars(DECL_LD Word after)
 
 #define __do_undo(m) LDFUNC(__do_undo, m)
 static inline void
-__do_undo(DECL_LD mark *m)
+__do_undo(DECL_LD const mark *m)
 { TrailEntry tt = tTop;
   TrailEntry mt = m->trailtop.as_ptr;
 
@@ -2416,7 +2417,7 @@ choice_type last_choice;
 #endif
 
 #define FRAME_FAILED		GO(deep_backtrack)
-#define CLAUSE_FAILED		GO(shallow_backtrack)
+#define CLAUSE_FAILED		GO(unify_backtrack)
 #define BODY_FAILED		GO(shallow_backtrack)
 #ifdef O_DEBUGGER
 #define TRACE_RETRY		VMH_GOTO(retry)
@@ -2521,7 +2522,7 @@ chp_chars(Choice ch)
 #endif
 
 
-int
+bool
 existingChoice(DECL_LD Choice ch)
 { if ( onStack(local, ch) && onStack(local, ch->frame) &&
        (int)ch->type >= 0 && (int)ch->type <= CHP_DEBUG )
@@ -2537,6 +2538,24 @@ existingChoice(DECL_LD Choice ch)
 }
 
 
+bool
+existingFrame(DECL_LD LocalFrame fr)
+{ for(;;)
+  { if ( !onStack(local, fr) )
+      return false;
+    if ( !isFrame(fr) )
+      return false;
+
+    if ( fr->parent )
+    { fr = fr->parent;
+    } else
+    { QueryFrame qf = queryOfFrame(fr);
+      return qf->magic == QID_MAGIC;
+    }
+  }
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 discardChoicesAfter() discards all choicepoints created  after fr, while
 calling  possible  hooks  on   the   frames.    If   the   `reason`   is
@@ -2702,18 +2721,17 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   size_t i, arity;
   Word ap;
   size_t lneeded;
-  static int top_initialized = false;
-  static struct clause clause;
-  static struct clause_ref cref;
 
-  if ( !top_initialized )
-  { clause.predicate = PROCEDURE_dc_call_prolog->definition;
-    clause.generation.erased = ~(gen_t)0;
-    clause.code_size = 1;
-    clause.codes[0] = encode(I_EXITQUERY);
-    cref.value.clause = &clause;
+  if ( !GD->clauses.top_clause )
+  { Clause cl = allocHeapOrHalt(sizeofClause(1));
 
-    top_initialized = true;
+    memset(cl, 0, sizeofClause(1));
+    cl->predicate = PROCEDURE_dc_call_prolog->definition;
+    cl->generation.erased = ~(gen_t)0;
+    cl->code_size = 1;
+    cl->codes[0] = encode(I_EXITQUERY);
+    GD->clauses.top_cref.value.clause = cl;
+    GD->clauses.top_clause = cl;	/* MT-safe as a init is a single thread */
   }
 
   DEBUG(2, { FunctorDef f = proc->definition->functor;
@@ -2763,7 +2781,7 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   top->parent        = NULL;
   top->predicate     = PROCEDURE_dc_call_prolog->definition;
   top->programPointer= NULL;
-  top->clause        = &cref;
+  top->clause        = &GD->clauses.top_cref;
 #ifdef O_PROFILE
   if ( LD->profile.active )
     top->prof_node = profCall(top->predicate);
@@ -2782,7 +2800,7 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   fr->parent         = top;
   setNextFrameFlags(fr, top);
   set(top, FR_HIDE_CHILDS);
-  fr->programPointer = clause.codes;
+  fr->programPointer = GD->clauses.top_clause->codes;
 
   DEBUG(CHK_SECURE, checkStacks(NULL));
   assert((uintptr_t)fli_context > (uintptr_t)environment_frame);
